--_________________________________________________________________________________________________
--                                                                                                |
--                                           |TDC core|                                           |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                      fmc_masterFIP_core                                        |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         fmc_masterFIP_core.vhd                                                            |
--                                                                                                |
-- Description  The masterFIP_core instantiates all the modules needed to establish WorldFIP      |
--              communication. Figure 1 shows the modules of the core.                            |
--                                                                                                |
--         _      ________________________________________________________________                |
--        | |    |                                             fmc_masterFIP_core |               |
--        |W|    |   _________________    ____________    ____________    ______  |               |
--        |O|    |  |                 |  | macrocycle |  |  silence   |  |      | |               |
--        |R| <--|  |  masterfip_tx   |  | time cnt   |  |  time cnt  |  |      | |               |
--        |L|    |   _________________|  |____________|  |____________|  |      | |               |
--        |D|    |   _________________    ____________    ____________   | fmc  | |               |
--        |F| -->|  |                 |  | turnaround |  |            |  |master| | <-processor-> |
--        |I|    |  | masterfip_rx    |  |  time cnt  |  |   resets   |  | FIP  | |               |
--        |P|    |  |_________________|  |____________|  |____________|  | csr  | |               |
--        |_|    |                                                       |      | |               |
--               |   _________________                                   |      | |               |
-- sync pulse -->|  |  ext sync pulse |                                  |      | |               |
--               |  |_________________|                                  |      | |               |
--               |   _______                                             |      | |               |
--      LEDs  <--|  |  aux  |                                            |      | |               |
--               |  |_______|                                            |______| |               |
--               |________________________________________________________________|               |
--                                                Figure 1                                        |
--                                                                                                |
--              The fmc_masterfip_csr module establishes the interface with the processor, usually|
--              a Mock Turtle core. The module has been generated through the wbgen2 application. |
--              Please consult the fmc_masterfip_csr.htm for the registers map.                   |
--                                                                                                |
--              The masterfip_tx is placing a complete WorldFIP frame on the bus.                 |
--              The masterfip_tx is ignorant of the frame type (ID_DAT/RT_DAT/RP_MSG etc..) or the|
--              macrocycle sequence, as well as of timing issues (it does not for example measure |
--              internally the turnaround time after the delivery of an ID_DAT); the processor    |
--              (Mock Turtle for example) is responsible for managing all these issues and        |
--              for providing to the masterfip_tx (through the fmc_masterfip_csr) the data to     |
--              serialise and a start pulse. The communication between the processor (MT) and the |
--              masterfip_tx is through a set of control (from the MT) and status (from the       |
--              masterfip_tx) registers. Upon a rising edge on the tx_start control pulse, the    |
--              masterfip_tx is copying all the data registers (tx_data_ctrl,                     |
--              tx_data_reg1..tx_data_reg66) and then it initiates a WorldFIP frame, starting with|
--              the FSS, the tx_data_ctrl and then the number of data bytes indicated by the      |
--              tx_bytes_num register; it finally adds the CRC and FES and signals the processor  |
--              through the status bit tx_stop.                                                   |
--               ______________________________________________________________________________   |
--              |_____FSS_____|__Ctrl__|_____________tx_data______________|_____CRC____|__FES__|  |
--                                                                                                |
--              <-- 2 byes --><-1byte-><---------  tx_bytes_num  --------><-- 2 byes --><-1byte-> |
--                                                                                                |
--              The masterfip_rx is retrieving a WorldFIP frame from the bus.                     |
--              Similar to the masterfip_tx, the masterfip_rx has no intelligence regarding the   |
--              macrocycle sequence; it is controlled and monitored by the processor through the  |
--              fmc_masterfip_csr and a set of control and status registers.                      |
--              As long as it is not under reset, the masterfip_rx is looking at the WorldFIP bus |
--              trying to identify the FSS sequence of a WorldFIP frame. Upon the FSS detection   |
--              it signals the processor (MT) through the status bit fss_received and continues   |
--              with following the rest of the frame. It stores the first byte after the FSS to   |
--              the rx_data_ctrl register and the rest of the bytes to the registers              |
--              rx_data_reg1..rx_data_reg66. Upon the detection of a FES the masterfip_rx is      |
--              checking the CRC of the frame and enables the status bit rx_frame_ok or           |
--              rx_frame_error accordingly. Upon the rx_frame_ok, the status register             |
--              rx_bytes_num indicates the number of bytes that were in the frame (this indicates |
--              the number of rx_data_regs and the number of bytes inside the last rx_data_reg to |
--              be retrieved by the processor).                                                   |
--               ______________________________________________________________________________   |
--              |_____FSS_____|__Ctrl__|_____________rx_data______________|_____CRC____|__FES__|  |
--                                                                                                |
--              <-- 2 byes --><-1byte-><---------  rx_bytes_num  --------><-- 2 byes --><-1byte-> |
--                                                                                                |
--              The modules regarding the ext_sync_pulse are counting the number of rising-edge   |
--              pulses that are arriving to the ext_sync input of the board and provide the result|
--              to the dedicated fmc_masterfip_csr register to be accessed by the processor (MT). |
--                                                                                                |
--              The modules regarding the macrocycle are counting the time of a macrocycle using  |
--              the 10ns input clock as well as the number of macrocycles since the               |
--              startup/a reset. Dedicated registers in the fmc_masterfip_csr provide the counters|
--              values to the processor.                                                          |
--                                                                                                |
--              The modules regarding the turnaround and silence time are counting the respective |
--              time using the 10ns clock. Dedicated registers in the fmc_masterfip_csr provide   |
--              the counters values to the processor.                                             |
--                                                                                                |
-- Authors      Evangelia Gousiou (Evangelia.Gousiou@cern.ch)                                     |
-- Date         07/2014                                                                           |
-- Version      v1                                                                                |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     07/2014  v1  EG  First version                                                             |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------

--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.mrfip_wbgen2_pkg.all;
use work.masterFIP_pkg.all;
use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.wrcore_pkg.all;
use work.genram_pkg.all;
use work.wf_package.all;


--=================================================================================================
--                                Entity declaration for fmc_masterFIP_core
--=================================================================================================
entity fmc_masterFIP_core is
  generic
    (g_span             : integer := 32;     -- address span in bus interfaces
     g_width            : integer := 32;     -- data width in bus interfaces
     values_for_simul   : boolean := FALSE); -- set to TRUE when instantiated in a test-bench
  port
    (-- Clock and reset
     clk_i              : in    std_logic;   -- only one clk domain
     rst_i              : in    std_logic;   -- PoR or PCIe reset, synched with the clk_i

     -- Bus Speed                            -- 31.25 Kbps: speed_b1 = 0, speed_b0 = 0
     speed_b0_i         : in    std_logic;   -- 1 Mbps    : speed_b1 = 0, speed_b0 = 1
     speed_b1_i         : in    std_logic;   -- 2.5 Mbps  : speed_b1 = 1, speed_b0 = 0
                                             -- 5 Mbps    : speed_b1 = 1, speed_b0 = 1
     -- One wire DS18B20U+ on the mezzanine
     onewire_b          : inout std_logic;   -- mezzanine 1-wire

     -- External synchronisation pulse transceiver
     ext_sync_term_en_o : out std_logic;     -- enable 50 Ohms termination of the pulse
     ext_sync_dir_o     : out std_logic;     -- transceiver direction
     ext_sync_oe_n_o    : out std_logic;     -- transceiver output enable negative
     ext_sync_tst_n_o   : out std_logic;     -- emulation of the LEMO input
     ext_sync_i         : in std_logic;      -- sync pulse

     -- Power supplies for the ADC
     adc_1v8_shdn_n_o   : out std_logic;
     adc_m5v_shdn_n_o   : out std_logic;
     adc_5v_en_n_o      : out std_logic;
     adc_prim_conn_n_o  : out std_logic;
     adc_sec_conn_n_o   : out std_logic;

     -- DAC configuration (to set OSC1 to its middle range where the stability is max)
     dac_cs_n_o         : out std_logic_vector(1 downto 0);
     dac_sclk_o         : out std_logic;
     dac_din_o          : out std_logic;

     -- FielDrive
     fd_rxcdn_i         : in  std_logic;
     fd_rxd_i           : in  std_logic;
     fd_txer_i          : in  std_logic;
     fd_wdgn_i          : in  std_logic;
     fd_rstn_o          : out std_logic;
     fd_txck_o          : out std_logic;
     fd_txd_o           : out std_logic;
     fd_txena_o         : out std_logic;

    -- WISHBONE classic bus interface
     wb_adr_i           : in  std_logic_vector(g_span-1 downto 0);
     wb_dat_i           : in  std_logic_vector(g_width-1 downto 0);
     wb_stb_i           : in  std_logic;
     wb_sel_i           : in  std_logic_vector(3 downto 0);
     wb_we_i            : in  std_logic;
     wb_cyc_i           : in  std_logic;
     wb_stall_o         : out std_logic;
     wb_ack_o           : out std_logic;
     wb_dat_o           : out std_logic_vector(g_width-1 downto 0);

     -- Aux, 8 debugging signals to pass to the higher levels
     aux_o              : out std_logic_vector(7 downto 0));
  end fmc_masterFIP_core;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of fmc_masterFIP_core is

  -- wbgen2 regs from/to mock turtle
  signal reg_to_mt                                            : t_mrfip_in_registers;
  signal reg_from_mt                                          : t_mrfip_out_registers;
  -- resets
  signal rst_n, core_rst, core_rst_n                          : std_logic;
  signal fd_host_rst                                          : std_logic;
  -- config
  signal speed                                                : std_logic_vector(1 downto 0);
  -- ext pulse
  signal ext_sync_p, ext_sync_oe                              : std_logic;
  signal ext_sync_p_cnt_rst, ext_sync_p_cnt_host_rst          : std_logic;
  signal ext_sync_p_cnt                                       : std_logic_vector(31 downto 0);
  -- counters
  signal macrocyc_load_p, turnar_load_p, silen_load_p         : std_logic;
  signal num_of_macrocyc_cnt                                  : std_logic_vector(31 downto 0);
  signal num_of_macrocyc_cnt_full, num_of_macrocyc_cnt_reinit : std_logic;
  signal macrocyc_cnt                                         : std_logic_vector(30 downto 0);
  signal macrocyc_cnt_zero_p                                  : std_logic;
  -- tx
  signal tx_completed_p, tx_completed                         : std_logic;
  signal tx_rst, fd_txd, fd_txck                              : std_logic;
  signal tx_frame                                             : tx_frame_t;
  -- rx
  signal rx_rst                                               : std_logic;
  signal rx_fss_received_p, rx_fss_received                   : std_logic;
  signal rx_frame_ok_p, rx_frame_ok, rx_crc_wrong_p           : std_logic;
  signal rx_crc_wrong, rx_byte_ready_p, fd_txena              : std_logic;
  signal rx_frame                                             : rx_frame_t;
  signal rx_byte                                              : std_logic_vector(7 downto 0);
  signal rx_byte_index                                        : std_logic_vector(15 downto 0);
  -- fd_wdgn, fd_txer, fd_rxcdn
  signal fd_wdg_tstamp, fd_txer_tstamp                        : std_logic_vector(30 downto 0);
  signal fd_wdg_sync, fd_wdg, fd_rxcd, fd_rxcdn_sync          : std_logic;
  signal fd_txer, fd_txer_p, fd_txer_cnt_reinit               : std_logic;
  -- one wire
  signal tmp_temper                                           : std_logic_vector(15 downto 0);
  signal tmp_id                                               : std_logic_vector(63 downto 0);
  signal onewire_read_p, pps_is_zero, pps_load_p              : std_logic;
  -- ext_sync
  signal ext_sync, ext_sync_filt                              : std_logic;
  -- LEDs
  signal sync_led, out_of_sync_led                            : std_logic;
  -- debug
  signal mf_dbg                                               : std_logic_vector(31 downto 0);
  signal mf_dbg_p, mf_dbg_p_ext, macrocyc_cnt_zero, macrocyc_cnt_twentyfive, macrocyc_cnt_twentyfive_p            : std_logic;
 

  -- chipscope
   -- component chipscope_ila
     -- port (
       -- CONTROL : inout std_logic_vector(35 downto 0);
       -- CLK     : in    std_logic;
       -- TRIG0   : in    std_logic_vector(31 downto 0);
       -- TRIG1   : in    std_logic_vector(31 downto 0);
       -- TRIG2   : in    std_logic_vector(31 downto 0);
       -- TRIG3   : in    std_logic_vector(31 downto 0));
   -- end component;
   -- component chipscope_icon
     -- port (CONTROL0 : inout std_logic_vector(35 downto 0));
   -- end component;
   -- signal CONTROL : std_logic_vector(35 downto 0);
   -- signal CLK     : std_logic;
   -- signal TRIG0   : std_logic_vector(31 downto 0);
   -- signal TRIG1   : std_logic_vector(31 downto 0);
   -- signal TRIG2   : std_logic_vector(31 downto 0);
   -- signal TRIG3   : std_logic_vector(31 downto 0);


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

---------------------------------------------------------------------------------------------------
--                                WBGEN2 REGS FROM/TO MOCK TURTLE                                --
---------------------------------------------------------------------------------------------------
  cmp_masterfip_csr: fmc_masterfip_csr
  port map
    (rst_n_i    => rst_n,
     clk_sys_i  => clk_i,
     wb_adr_i   => wb_adr_i(9 downto 2),
     wb_dat_i   => wb_dat_i,
     wb_dat_o   => wb_dat_o,
     wb_cyc_i   => wb_cyc_i,
     wb_sel_i   => wb_sel_i,
     wb_stb_i   => wb_stb_i,
     wb_we_i    => wb_we_i,
     wb_ack_o   => wb_ack_o,
     wb_stall_o => wb_stall_o,
     regs_i     => reg_to_mt,
     regs_o     => reg_from_mt);


---------------------------------------------------------------------------------------------------
--                                    ADC SUPPLIES & RELAYS                                      --
---------------------------------------------------------------------------------------------------
  adc_1v8_shdn_n_o   <= reg_from_mt.adc_1v8_shdn_n_o;
  adc_m5v_shdn_n_o   <= reg_from_mt.adc_m5v_shdn_n_o;
  adc_5v_en_n_o      <= reg_from_mt.adc_5v_en_n_o;
  adc_sec_conn_n_o   <= '0' when reg_from_mt.adc_sec_conn_n_o  = '0' else 'Z';
  adc_prim_conn_n_o  <= '0' when reg_from_mt.adc_prim_conn_n_o = '0' else 'Z';


---------------------------------------------------------------------------------------------------
--                                             speed                                            --
---------------------------------------------------------------------------------------------------
  speed              <= speed_b1_i & speed_b0_i;
  reg_to_mt.speed_i  <= speed;


---------------------------------------------------------------------------------------------------
--                                             resets                                            --
---------------------------------------------------------------------------------------------------
  rst_n      <= not rst_i;                       -- PoR or PCIe reset
  core_rst   <= reg_from_mt.rst_core_o or rst_i; -- PoR or PCIe reset or reset from WRNC
  core_rst_n <= not core_rst;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- FIELDRIVE reset generation of length: 2 WorldFIP-clk-cycles of the lowest speed 31.25Kbps
  cmp_fd_rst_generate: gc_extend_pulse
  generic map (g_width => 6400)
  port map
    (clk_i      => clk_i,
     rst_n_i    => core_rst_n,
     pulse_i    => reg_from_mt.rst_fd_o, -- monostable: 1-clk-tick-long pulse
     extended_o => fd_host_rst);
  --  --  --  --  --  --  --  --  --  --  --
  fd_rstn_o <= not fd_host_rst;


---------------------------------------------------------------------------------------------------
--                                         external sync                                         --
---------------------------------------------------------------------------------------------------

  ext_sync_tst_n_o   <= '0' when reg_from_mt.ext_sync_tst_n_o  = '0' else 'Z';
  ext_sync_dir_o     <= reg_from_mt.ext_sync_dir_o;
  ext_sync_term_en_o <= reg_from_mt.ext_sync_term_en_o;
  ext_sync_oe_n_o    <= reg_from_mt.ext_sync_oe_n_o;

  -- input synchronizer of the ext_sync_i signal
  cmp_ext_sync_sync: gc_sync_ffs
  port map
    (clk_i     => clk_i,
     rst_n_i   => core_rst_n,
     data_i    => ext_sync_i,
     synced_o  => ext_sync);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- deglitch filter
  cmp_ext_sync_glitch_filt: gc_glitch_filt
  generic map(g_len => 10) -- glitches up to 100ns are ignored;
  port map                 -- Note that the filter adds a 100ns delay to the ext_sync signal
    (clk_i   => clk_i,
     rst_n_i => core_rst_n,
     dat_i   => ext_sync,
     dat_o   => ext_sync_filt);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- rising edge detection on the deglitched signal
  cmp_ext_sync_p_detect: gc_sync_ffs
  port map
    (clk_i    => clk_i,
     rst_n_i  => core_rst_n,
     data_i   => ext_sync_filt,
     ppulse_o => ext_sync_p);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- counter of the number of rising edges
  cmp_ext_sync_p_cnt:incr_counter
  generic map(g_counter_lgth => 32)
  port map
    (clk_i            => clk_i,
     counter_incr_i   => ext_sync_p,
     counter_reinit_i => ext_sync_p_cnt_rst,
     counter_o        => reg_to_mt.ext_sync_p_cnt_i);
  --  --  --  --  --  --  --  --  --  --  --
  ext_sync_p_cnt_rst <= ext_sync_p_cnt_host_rst or core_rst;


---------------------------------------------------------------------------------------------------
--                                       macrocycle counter                                      --
---------------------------------------------------------------------------------------------------
-- Regarding synchronisation, the masterFIP application can work in two modes:
-- using an internal counter that counts each macrocycle based on the local oscillator, or
-- using the ext_sync_i pulse to signal the beginning of each macrocycle. There is also the option
-- that when the external pulse does not arrive, the internal counter takes over the synchronisation.
-- In an application where the external synchronization pulse is expected for every macrocycle,
-- the macrocyc_lgth is set by the sw application slightly larger than the external synchronization
-- period; therefore the macrocycle counter should not be reaching zero; if it reaches zero,
-- it would mean that the external synchronization pulse did not arrive in time and a new macrocycle,
-- (slightly longer) will start automatically based on this internal counter.

  -- counter counting the macrocycle time;
  cmp_macrocycle_time_cnt: decr_counter
  generic map(width   => 31)
  port map
    (clk_i             => clk_i,
     rst_i             => core_rst,
     counter_load_i    => macrocyc_load_p,
     counter_top_i     => reg_from_mt.macrocyc_lgth_o,
     counter_o         => macrocyc_cnt,
     counter_is_zero_o => macrocyc_cnt_zero_p);
  --  --  --  --  --  --  --  --  --  --  --
  reg_to_mt.macrocyc_time_cnt_i <= macrocyc_cnt;

  macrocyc_load_p <= macrocyc_cnt_zero_p or reg_from_mt.macrocyc_start_o or ext_sync_p;
  -- note: macrocyc_start_o is a monostable, 1-clk-tick-long pulse 
 

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- counter counting the number of macrocycles;
  -- being a 32-bit counter, for the fastest application of 20ms macrocycle, the counter can 
  -- keep counting for 994 days; when it fills up it would restart from 0.
  cmp_macrocycles_cnt:incr_counter
  generic map(g_counter_lgth => 32)
  port map
    (clk_i             => clk_i,
     counter_incr_i    => macrocyc_cnt_zero_p,
     counter_reinit_i  => num_of_macrocyc_cnt_reinit,
     counter_is_full_o => num_of_macrocyc_cnt_full,
     counter_o         => reg_to_mt.macrocyc_num_cnt_i);
  --  --  --  --  --  --  --  --  --  --  --
  num_of_macrocyc_cnt_reinit <= core_rst_n or num_of_macrocyc_cnt_full;


---------------------------------------------------------------------------------------------------
--                                       turnaround counter                                      --
---------------------------------------------------------------------------------------------------
-- counter counting the turnaround time i.e. the time after the end of transmission of a frame
-- (tx_completed_p), or after the end of reception of a frame (rx_frame_ok_p/ rx_crc_wrong_p)
-- and before the transmission of a new frame.
-- Note that the Mock Turtle is using the turnar_cnt rather than the 1-clk-tick long counter_is_zero

  -- turnaround counter
  cmp_turnaround_cnt: decr_counter
  generic map(width   => 31)
  port map
    (clk_i             => clk_i,
     rst_i             => core_rst,
     counter_load_i    => turnar_load_p,
     counter_top_i     => reg_from_mt.turnar_lgth_o,
     counter_o         => reg_to_mt.turnar_time_cnt_i,
     counter_is_zero_o => open);
  --  --  --  --  --  --  --  --  --  --  --
  turnar_load_p <= tx_completed_p or rx_frame_ok_p or rx_crc_wrong_p or reg_from_mt.turnar_start_o;
  -- note: turnar_start_o is a monostable, 1-clk-tick-long pulse 


--------------------------------------------------------------------------------------------------
--                                        silence counter                                       --
--------------------------------------------------------------------------------------------------
-- counter counting the silence time i.e. the maximum amount of time that the masterFIP waits for
-- a response frame; the counting starts after the transmission of a frame sent by the master
-- (tx_completed_p) or after the reception of a frame (rx_frame_ok_p/ rx_crc_wrong_p) for the
-- case of a RP_DAT_MSG that would be followed by RP_FIN.

  cmp_silence_cnt: decr_counter
  generic map(width   => 31)
  port map
    (clk_i             => clk_i,
     rst_i             => core_rst,
     counter_load_i    => silen_load_p,
     counter_top_i     => reg_from_mt.silen_lgth_o,
     counter_o         => reg_to_mt.silen_time_cnt_i,
     counter_is_zero_o => open);
  --  --  --  --  --  --  --  --  --  --  --
  silen_load_p <= tx_completed_p or rx_frame_ok_p or rx_crc_wrong_p or reg_from_mt.silen_start_o;
  -- note: turnar_start_o is a monostable, 1-clk-tick-long pulse 


--------------------------------------------------------------------------------------------------
--                                        signals for rx                                        --
--------------------------------------------------------------------------------------------------
-- The receiver rx is disabled when a frame is being transmitted (fd_txena active).
-- Note that the reg_from_mt.rx_ctrl_rst_o is a monostable, 1-clk-tick-long pulse
  rx_rst <= reg_from_mt.rx_ctrl_rst_o or fd_txena;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- registering the number of received bytes upon rx_frame_ok_p
  p_rx_bytes_num : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if (core_rst_n = '0' or rx_rst = '1') then
        reg_to_mt.rx_stat_bytes_num_i  <= (others => '0');
        reg_to_mt.rx_stat_frame_ok_i   <= '0';
      else
        if rx_frame_ok_p = '1' then
          reg_to_mt.rx_stat_frame_ok_i   <= rx_frame_ok_p;
          reg_to_mt.rx_stat_bytes_num_i  <= rx_byte_index - 3; -- data payload, without FSS, CTRL, CRC, FES
        end if;
      end if;
    end if;
  end process;
  
  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- extending the rx_fss_received_p until a core_rst or rx_rst is received
  p_rx_fss_received_extend : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(core_rst_n = '0' or rx_rst = '1') then
        reg_to_mt.rx_stat_pream_ok_i   <= '0';
      else
        if rx_fss_received_p = '1' then
          reg_to_mt.rx_stat_pream_ok_i <= '1';
        end if;
      end if;
    end if;
  end process;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- extending the rx_crc_wrong_p  until a core_rst or rx_rst is received
  p_rx_crc_wrong_extend : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(core_rst_n = '0' or rx_rst = '1') then
        reg_to_mt.rx_stat_frame_crc_err_i   <= '0';
      else
        if rx_crc_wrong_p = '1' then
          reg_to_mt.rx_stat_frame_crc_err_i <= '1';
        end if;
      end if;
    end if;
  end process;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- counter of frames with crc errors since the startup or a core reset (not on every macrocycle)
  cmp_rx_crc_err_cnt:incr_counter
  generic map(g_counter_lgth => 32)
  port map
    (clk_i            => clk_i,
     counter_incr_i   => rx_crc_wrong_p,
     counter_reinit_i => core_rst_n,
     counter_o        => reg_to_mt.rx_stat_crc_err_cnt_i);


---------------------------------------------------------------------------------------------------
--                                               rx                                              --
---------------------------------------------------------------------------------------------------
  cmp_masterfip_rx: masterfip_rx
  port map
    (clk_i                 => clk_i,
     rst_i                 => core_rst,
     speed_i               => speed,
     fd_rxd_a_i            => fd_rxd_i,
     rx_rst_i              => rx_rst,        -- reset from the WRNC or reset while transmitting
  -------------------------------------------------------------
     rx_byte_index_o       => rx_byte_index, -- not currently used
     rx_word_index_o       => reg_to_mt.rx_stat_curr_word_indx_i,
     rx_ctrl_byte_o        => reg_to_mt.rx_payld_ctrl_i,
     rx_frame_o            => rx_frame,
     rx_byte_o             => rx_byte,
     rx_byte_ready_p_o     => rx_byte_ready_p,
     rx_fss_crc_fes_ok_p_o => rx_frame_ok_p,
     rx_fss_received_p_o   => rx_fss_received_p,
     rx_crc_wrong_p_o      => rx_crc_wrong_p);
  -------------------------------------------------------------


---------------------------------------------------------------------------------------------------
--                                               tx                                              --
---------------------------------------------------------------------------------------------------

  cmp_masterfip_tx: masterfip_tx
  port map
    (clk_i           => clk_i,
     rst_i           => tx_rst,
     speed_i         => speed,
     tx_bytes_num_i  => reg_from_mt.tx_ctrl_bytes_num_o, -- number of bytes to serialize; sampled upon tx_start_p
     tx_start_p_i    => reg_from_mt.tx_ctrl_start_o,     -- monostable, 1-clk-tick-long pulse
     tx_frame_i      => tx_frame,
     tx_ctrl_byte_i  => reg_from_mt.tx_payld_ctrl_o,
  -------------------------------------------------------------
     tx_byte_index_o => reg_to_mt.tx_stat_curr_byte_indx_i, -- index of current byte being serialized  -- up to 263 bytes
     tx_end_p_o      => tx_completed_p,
     tx_data_o       => fd_txd,
     tx_enable_o     => fd_txena,
     tx_clk_o        => fd_txck);
  -------------------------------------------------------------

  tx_rst     <= core_rst or reg_from_mt.tx_ctrl_rst_o;
--  fd_txena_o <= fd_txena;
  fd_txd_o   <= fd_txd;
  fd_txck_o  <= fd_txck;

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- delay for 1 cycle the fd_txena
  fd_txena_delay : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(tx_rst = '1') then
         fd_txena_o <= '0';
      else
         fd_txena_o <= fd_txena;
      end if;
    end if;
  end process;
  reg_to_mt.tx_stat_ena_i <= fd_txena;

--------------------------------------------------------------------------------------------------
--                                        signals for tx                                        --
--------------------------------------------------------------------------------------------------
  -- extending the tx_completed_p until a core_rst or rx_rst is received
  -- note: reg_from_mt.tx_ctrl_rst_o and reg_from_mt.tx_ctrl_start_o are monostables,
  -- 1-clk-tick-long pulses

  p_tx_completed_extend : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(core_rst_n = '0' or reg_from_mt.tx_ctrl_rst_o = '1' or reg_from_mt.tx_ctrl_start_o = '1') then
        reg_to_mt.tx_stat_stop_i   <= '0';
      else
        if tx_completed_p = '1' then
          reg_to_mt.tx_stat_stop_i <= '1'; -- stays active until a tx_rst or core_rst or tx_start_p
        end if;
      end if;
    end if;
  end process;


---------------------------------------------------------------------------------------------------
--                                    FielDrive TXERR, WDGN, CDN                                 --
---------------------------------------------------------------------------------------------------
-- The fd_wdgn_i is activated when the FielDrive detects TXD activity > 1024 bytes long;
-- it is deactivated with a FielDrive reset.
-- The following processes provide to the MT the macrocycle number when the fd_wdgn_i was activated.
-- Note that it is the logic running on the MT that is responsible for activating a fd_rst_o upon
-- the detection of a fd_wdgn_i.
 
  -- synch of the incoming fd_wdgn_i signal
  cmp_fd_wdgn_sync: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map
    (clk_i    => clk_i,
     rst_n_i  => rst_n,
     data_i   => fd_wdg,
     synced_o => fd_wdg_sync);

     fd_wdg <= not fd_wdgn_i;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- process that registers at the same time the activation of fd_wdgn_i and the macrocycle_cnt
  -- upon the activation; the signals are reset upon a core_rst_n or a fd_host_rstn
  p_fd_wdgn_capture : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(core_rst = '1' or fd_host_rst = '1') then
        reg_to_mt.fd_wdg_i          <= '0';
        reg_to_mt.fd_wdg_tstamp_i   <= (others => '0');
      else
        if fd_wdg_sync = '1' then
          reg_to_mt.fd_wdg_i        <= '1';
          reg_to_mt.fd_wdg_tstamp_i <= '0' & macrocyc_cnt;
        end if;
      end if;
    end if;
  end process;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- synch of the incoming fd_rxcdn_i signal
  cmp_fd_rxcdn_sync: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map
    (clk_i    => clk_i,
     rst_n_i  => rst_n,
     data_i   => fd_rxcd,
     synced_o => fd_rxcdn_sync);

     fd_rxcd           <= not fd_rxcdn_i;
     reg_to_mt.fd_cd_i <= fd_rxcdn_sync;
  
  
--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
-- The fd_txerr_i is activated when the FielDrive detects an overload/underload; for example if

  -- edge detection on the fd_txer_i signal
  cmp_fd_txer_i_pulse_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map
    (clk_i    => clk_i,
     rst_n_i  => rst_n,
     data_i   => fd_txer_i,
     ppulse_o => fd_txer_p);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- counter counting the number of fd_txerr_p
  cmp_fd_txer_cnt:incr_counter
  generic map(g_counter_lgth => 32)
  port map
    (clk_i             => clk_i,
     counter_incr_i    => fd_txer_p,
     counter_reinit_i  => fd_txer_cnt_reinit,
     counter_is_full_o => open,
     counter_o         => reg_to_mt.fd_txer_cnt_i);
  --  --  --  --  --  --  --  --  --  --  --
  fd_txer_cnt_reinit <= '1' when core_rst_n = '0' or macrocyc_load_p = '1' or fd_host_rst = '1'
                   else '0';

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- process that registers the macrocycle_cnt value upon the last fd_txer_p of a macrocycle
  p_fd_txer_capture : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(core_rst_n = '0' or macrocyc_load_p = '1' or fd_host_rst = '1') then
        fd_txer                      <= '1';
        reg_to_mt.fd_txer_tstamp_i   <= (others => '0');
      else
        if fd_txer_p = '1' then
          fd_txer                    <= '0';
          reg_to_mt.fd_txer_tstamp_i <= '0' & macrocyc_cnt;
        end if;
      end if;
    end if;
  end process;



---------------------------------------------------------------------------------------------------
--                                              DAC                                              --
---------------------------------------------------------------------------------------------------
-- Communication with the SPEC DAC AD5662BRM2 that controls the 20 MHz OSC2 which is the source of
-- the design's timing. Setting the DAC at its middle range maximized the oscillator's stability;
-- this is important for free-running applications that are not using the ext_sync_i
  cmp_dac_config : spec_serial_dac_arb
  generic map
    (g_invert_sclk    => false,
     g_num_extra_bits => 8)
  port map
    (clk_i         => clk_i,
     rst_n_i       => core_rst_n,
     val1_i        => reg_from_mt.dac_config_value_o,
     load1_i       => reg_from_mt.dac_config_load_o, -- monostable: 1-clk-tick-long pulse
     val2_i        => (others => '0'),
     load2_i       => '0',
     dac_cs_n_o(0) => dac_cs_n_o(0),
     dac_cs_n_o(1) => dac_cs_n_o(1),
     dac_sclk_o    => dac_sclk_o,
     dac_din_o     => dac_din_o);



---------------------------------------------------------------------------------------------------
--                                       DS18B20U one wire                                       --
---------------------------------------------------------------------------------------------------
-- Communication with the 1-wire DS18B20U+ for the unique ID and temperature reading;
-- rather than sw bit-banging the whole 1-wire communication is implemented in vhdl, to simplify
-- drivers. Note that a temperature reading is provided every second, with the first one a couple
-- of sec after the board power-up/ reset.

  cmp_onewire: gc_ds182x_interface
  generic map (freq => 100)
  port map
    (clk_i     => clk_i,
     rst_n_i   => core_rst_n,
     onewire_b => onewire_b,
     id_o      => tmp_id,
     temper_o  => tmp_temper,
     id_read_o => onewire_read_p,
     pps_p_i   => pps_is_zero);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- pps generator based on the 100 MHz clk
  cmp_pps_gen: wf_decr_counter
  generic map(g_counter_lgth   => 27)
  port map
    (uclk_i            => clk_i,
     counter_rst_i     => core_rst,
     counter_decr_i    => '1',
     counter_load_i    => pps_load_p,
     counter_top_i     => "101111101011110000100000000", -- 100'000'000
     counter_is_zero_o => pps_is_zero);
  --  --  --  --  --  --  --  --  --  --  --
  pps_load_p <= pps_is_zero; -- looping

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- registering of the read values
  reg_reading: process(clk_i)
  begin
    if rising_edge(clk_i) then
      reg_to_mt.ds1820_temper_i   <= (others => '0');
      reg_to_mt.ds1820_id_lsb_i   <= (others => '0');
      reg_to_mt.ds1820_id_msb_i   <= (others => '0');
      if(onewire_read_p = '1') then
        reg_to_mt.ds1820_temper_i <= tmp_temper;
        reg_to_mt.ds1820_id_lsb_i <= tmp_id (31 downto 0);
        reg_to_mt.ds1820_id_msb_i <= tmp_id (63 downto 32);
      end if;
    end if;
  end process;



---------------------------------------------------------------------------------------------------
--                                          LEDs & aux                                           --
---------------------------------------------------------------------------------------------------
  cmp_out_of_sync_led: gc_extend_pulse
  generic map
    (g_width    => 1000000)
  port map
    (clk_i      => clk_i,
     rst_n_i    => core_rst_n,
     pulse_i    => macrocyc_cnt_zero_p,
     extended_o => out_of_sync_led);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  cmp_sync_led: gc_extend_pulse
  generic map
    (g_width    => 10000)--1000000
  port map
    (clk_i      => clk_i,
     rst_n_i    => core_rst_n,
     pulse_i    => ext_sync_p,
     extended_o => sync_led);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --

  cmp_tmp_ext: gc_extend_pulse
  generic map
    (g_width    => 10000)
  port map
    (clk_i      => clk_i,
     rst_n_i    => core_rst_n,
     pulse_i    => macrocyc_cnt_twentyfive_p,
     extended_o => macrocyc_cnt_twentyfive);

     macrocyc_cnt_twentyfive_p <= '1' when macrocyc_cnt = "0000000000000000000000000011001" else '0';
     aux_o(7) <= out_of_sync_led;
  aux_o(6) <= mf_dbg_p_ext;
  aux_o(5) <= reg_from_mt.dbg_o(5);
  aux_o(4) <= reg_from_mt.dbg_o(4); --rx_byte_ready_p;
  aux_o(3) <= ext_sync_p_cnt_rst;--mf_dbg(3); --macrocyc_cnt_zero_p;
  aux_o(2) <= reg_from_mt.macrocyc_start_o;--(2); --tx_completed;
  aux_o(1) <= macrocyc_cnt_twentyfive;--mf_dbg(1);
  aux_o(0) <= sync_led;--mf_dbg(0); --sync_led;


---------------------------------------------------------------------------------------------------
--                                             Sigh:-/                                           --
---------------------------------------------------------------------------------------------------
  -- tx regs
  tx_frame(0)  <= reg_from_mt.tx_payld_reg1_o;
  tx_frame(1)  <= reg_from_mt.tx_payld_reg2_o;
  tx_frame(2)  <= reg_from_mt.tx_payld_reg3_o;
  tx_frame(3)  <= reg_from_mt.tx_payld_reg4_o;
  tx_frame(4)  <= reg_from_mt.tx_payld_reg5_o;
  tx_frame(5)  <= reg_from_mt.tx_payld_reg6_o;
  tx_frame(6)  <= reg_from_mt.tx_payld_reg7_o;
  tx_frame(7)  <= reg_from_mt.tx_payld_reg8_o;
  tx_frame(8)  <= reg_from_mt.tx_payld_reg9_o;
  tx_frame(9)  <= reg_from_mt.tx_payld_reg10_o;
  tx_frame(10) <= reg_from_mt.tx_payld_reg11_o;
  tx_frame(11) <= reg_from_mt.tx_payld_reg12_o;
  tx_frame(12) <= reg_from_mt.tx_payld_reg13_o;
  tx_frame(13) <= reg_from_mt.tx_payld_reg14_o;
  tx_frame(14) <= reg_from_mt.tx_payld_reg15_o;
  tx_frame(15) <= reg_from_mt.tx_payld_reg16_o;
  tx_frame(16) <= reg_from_mt.tx_payld_reg17_o;
  tx_frame(17) <= reg_from_mt.tx_payld_reg18_o;
  tx_frame(18) <= reg_from_mt.tx_payld_reg19_o;
  tx_frame(19) <= reg_from_mt.tx_payld_reg20_o;
  tx_frame(20) <= reg_from_mt.tx_payld_reg21_o;
  tx_frame(21) <= reg_from_mt.tx_payld_reg22_o;
  tx_frame(22) <= reg_from_mt.tx_payld_reg23_o;
  tx_frame(23) <= reg_from_mt.tx_payld_reg24_o;
  tx_frame(24) <= reg_from_mt.tx_payld_reg25_o;
  tx_frame(25) <= reg_from_mt.tx_payld_reg26_o;
  tx_frame(26) <= reg_from_mt.tx_payld_reg27_o;
  tx_frame(27) <= reg_from_mt.tx_payld_reg28_o;
  tx_frame(28) <= reg_from_mt.tx_payld_reg29_o;
  tx_frame(29) <= reg_from_mt.tx_payld_reg30_o;
  tx_frame(30) <= reg_from_mt.tx_payld_reg31_o;
  tx_frame(31) <= reg_from_mt.tx_payld_reg32_o;
  tx_frame(32) <= reg_from_mt.tx_payld_reg33_o;
  tx_frame(33) <= reg_from_mt.tx_payld_reg34_o;
  tx_frame(34) <= reg_from_mt.tx_payld_reg35_o;
  tx_frame(35) <= reg_from_mt.tx_payld_reg36_o;
  tx_frame(36) <= reg_from_mt.tx_payld_reg37_o;
  tx_frame(37) <= reg_from_mt.tx_payld_reg38_o;
  tx_frame(38) <= reg_from_mt.tx_payld_reg39_o;
  tx_frame(39) <= reg_from_mt.tx_payld_reg40_o;
  tx_frame(40) <= reg_from_mt.tx_payld_reg41_o;
  tx_frame(41) <= reg_from_mt.tx_payld_reg42_o;
  tx_frame(42) <= reg_from_mt.tx_payld_reg43_o;
  tx_frame(43) <= reg_from_mt.tx_payld_reg44_o;
  tx_frame(44) <= reg_from_mt.tx_payld_reg45_o;
  tx_frame(45) <= reg_from_mt.tx_payld_reg46_o;
  tx_frame(46) <= reg_from_mt.tx_payld_reg47_o;
  tx_frame(47) <= reg_from_mt.tx_payld_reg48_o;
  tx_frame(48) <= reg_from_mt.tx_payld_reg49_o;
  tx_frame(49) <= reg_from_mt.tx_payld_reg50_o;
  tx_frame(50) <= reg_from_mt.tx_payld_reg51_o;
  tx_frame(51) <= reg_from_mt.tx_payld_reg52_o;
  tx_frame(52) <= reg_from_mt.tx_payld_reg53_o;
  tx_frame(53) <= reg_from_mt.tx_payld_reg54_o;
  tx_frame(54) <= reg_from_mt.tx_payld_reg55_o;
  tx_frame(55) <= reg_from_mt.tx_payld_reg56_o;
  tx_frame(56) <= reg_from_mt.tx_payld_reg57_o;
  tx_frame(57) <= reg_from_mt.tx_payld_reg58_o;
  tx_frame(58) <= reg_from_mt.tx_payld_reg59_o;
  tx_frame(59) <= reg_from_mt.tx_payld_reg60_o;
  tx_frame(60) <= reg_from_mt.tx_payld_reg61_o;
  tx_frame(61) <= reg_from_mt.tx_payld_reg62_o;
  tx_frame(62) <= reg_from_mt.tx_payld_reg63_o;
  tx_frame(63) <= reg_from_mt.tx_payld_reg64_o;
  tx_frame(64) <= reg_from_mt.tx_payld_reg65_o;
  tx_frame(65) <= reg_from_mt.tx_payld_reg66_o;
  tx_frame(66) <= reg_from_mt.tx_payld_reg67_o;

  -- rx regs
  reg_to_mt.rx_payld_reg1_i  <= rx_frame(0);
  reg_to_mt.rx_payld_reg2_i  <= rx_frame(1);
  reg_to_mt.rx_payld_reg3_i  <= rx_frame(2);
  reg_to_mt.rx_payld_reg4_i  <= rx_frame(3);
  reg_to_mt.rx_payld_reg5_i  <= rx_frame(4);
  reg_to_mt.rx_payld_reg6_i  <= rx_frame(5);
  reg_to_mt.rx_payld_reg7_i  <= rx_frame(6);
  reg_to_mt.rx_payld_reg8_i  <= rx_frame(7);
  reg_to_mt.rx_payld_reg9_i  <= rx_frame(8);
  reg_to_mt.rx_payld_reg10_i <= rx_frame(9);
  reg_to_mt.rx_payld_reg11_i <= rx_frame(10);
  reg_to_mt.rx_payld_reg12_i <= rx_frame(11);
  reg_to_mt.rx_payld_reg13_i <= rx_frame(12);
  reg_to_mt.rx_payld_reg14_i <= rx_frame(13);
  reg_to_mt.rx_payld_reg15_i <= rx_frame(14);
  reg_to_mt.rx_payld_reg16_i <= rx_frame(15);
  reg_to_mt.rx_payld_reg17_i <= rx_frame(16);
  reg_to_mt.rx_payld_reg18_i <= rx_frame(17);
  reg_to_mt.rx_payld_reg19_i <= rx_frame(18);
  reg_to_mt.rx_payld_reg20_i <= rx_frame(19);
  reg_to_mt.rx_payld_reg21_i <= rx_frame(20);
  reg_to_mt.rx_payld_reg22_i <= rx_frame(21);
  reg_to_mt.rx_payld_reg23_i <= rx_frame(22);
  reg_to_mt.rx_payld_reg24_i <= rx_frame(23);
  reg_to_mt.rx_payld_reg25_i <= rx_frame(24);
  reg_to_mt.rx_payld_reg26_i <= rx_frame(25);
  reg_to_mt.rx_payld_reg27_i <= rx_frame(26);
  reg_to_mt.rx_payld_reg28_i <= rx_frame(27);
  reg_to_mt.rx_payld_reg29_i <= rx_frame(28);
  reg_to_mt.rx_payld_reg30_i <= rx_frame(29);
  reg_to_mt.rx_payld_reg31_i <= rx_frame(30);
  reg_to_mt.rx_payld_reg32_i <= rx_frame(31);
  reg_to_mt.rx_payld_reg33_i <= rx_frame(32);
  reg_to_mt.rx_payld_reg34_i <= rx_frame(33);
  reg_to_mt.rx_payld_reg35_i <= rx_frame(34);
  reg_to_mt.rx_payld_reg36_i <= rx_frame(35);
  reg_to_mt.rx_payld_reg37_i <= rx_frame(36);
  reg_to_mt.rx_payld_reg38_i <= rx_frame(37);
  reg_to_mt.rx_payld_reg39_i <= rx_frame(38);
  reg_to_mt.rx_payld_reg40_i <= rx_frame(39);
  reg_to_mt.rx_payld_reg41_i <= rx_frame(40);
  reg_to_mt.rx_payld_reg42_i <= rx_frame(41);
  reg_to_mt.rx_payld_reg43_i <= rx_frame(42);
  reg_to_mt.rx_payld_reg44_i <= rx_frame(43);
  reg_to_mt.rx_payld_reg45_i <= rx_frame(44);
  reg_to_mt.rx_payld_reg46_i <= rx_frame(45);
  reg_to_mt.rx_payld_reg47_i <= rx_frame(46);
  reg_to_mt.rx_payld_reg48_i <= rx_frame(47);
  reg_to_mt.rx_payld_reg49_i <= rx_frame(48);
  reg_to_mt.rx_payld_reg50_i <= rx_frame(49);
  reg_to_mt.rx_payld_reg51_i <= rx_frame(50);
  reg_to_mt.rx_payld_reg52_i <= rx_frame(51);
  reg_to_mt.rx_payld_reg53_i <= rx_frame(52);
  reg_to_mt.rx_payld_reg54_i <= rx_frame(53);
  reg_to_mt.rx_payld_reg55_i <= rx_frame(54);
  reg_to_mt.rx_payld_reg56_i <= rx_frame(55);
  reg_to_mt.rx_payld_reg57_i <= rx_frame(56);
  reg_to_mt.rx_payld_reg58_i <= rx_frame(57);
  reg_to_mt.rx_payld_reg59_i <= rx_frame(58);
  reg_to_mt.rx_payld_reg60_i <= rx_frame(59);
  reg_to_mt.rx_payld_reg61_i <= rx_frame(60);
  reg_to_mt.rx_payld_reg62_i <= rx_frame(61);
  reg_to_mt.rx_payld_reg63_i <= rx_frame(62);
  reg_to_mt.rx_payld_reg64_i <= rx_frame(63);
  reg_to_mt.rx_payld_reg65_i <= rx_frame(64);
  reg_to_mt.rx_payld_reg66_i <= rx_frame(65);
  reg_to_mt.rx_payld_reg67_i <= rx_frame(66);  
  
---------------------------------------------------------------------------------------------------
--       CHIPSCOPE       --
---------------------------------------------------------------------------------------------------
  -- chipscope_ila_1 : chipscope_ila
  -- port map (
    -- CONTROL <= CONTROL;
    -- CLK     <= clk_i;
    -- TRIG0   <= TRIG0;
    -- TRIG1   <= TRIG1;
    -- TRIG2   <= TRIG2;
    -- TRIG3   <= TRIG3);

  -- chipscope_icon_1 : chipscope_icon
  -- port map ( CONTROL0 <= CONTROL);

  -- TRIG0(8 downto 0)   <= reg_from_mt.tx_ctrl_bytes_num_o;
  -- TRIG0(18 downto 11) <= tx_ctrl_byte;
  -- TRIG0(27 downto 19) <= reg_to_mt.tx_stat_curr_byte_indx_i;
  -- TRIG0(28)  <= tx_completed_p;
  -- TRIG0(29)  <= fd_txena;
  -- TRIG0(30)  <= fd_txd;
  -- TRIG0(31)  <= fd_txck;
  -- TRIG1      <= tx_frame(0);
  -- TRIG1(15 downto 8)  <= rx_byte_index;
  -- TRIG2(8 downto 0)   <= rx_byte_index;
  -- TRIG2(9)   <= rx_fss_received_p;
  -- TRIG2(10)  <= rx_byte_ready_p;
  -- TRIG2(18 downto 11) <= rx_byte;
  -- TRIG2(26 downto 19) <= rx_ctrl_byte;
  -- TRIG2(27)  <= rx_frame_ok_p;
  -- TRIG2(28)  <= rx_crc_wrong_p;
  -- TRIG2(29)  <= core_rst;
  -- TRIG2(31 downto 30) <= speed_b1_i & speed_b0_i;

end rtl;
----------------------------------------------------------------------------------------------------
--  architecture ends
----------------------------------------------------------------------------------------------------
